@@ -3,7 +3,7 @@
  FILE: icaltimezone.c
  CREATOR: Damon Chaplin 15 March 2001
 
- $Id: icaltimezone.c,v 1.23.2.1 2006/05/12 07:11:58 ireneh Exp $
+ $Id: icaltimezone.c,v 1.26 2006/06/16 10:33:12 tml Exp $
  $Locker:  $
 
  (C) COPYRIGHT 2001, Damon Chaplin
@@ -44,6 +44,13 @@
 #include <sys/stat.h>
 #include <windows.h>
 #define snprintf _snprintf
+/* Undef the similar macro from pthread.h, it doesn't check if
+ * gmtime() returns NULL.
+ */
+#undef gmtime_r
+
+/* The gmtime() in Microsoft's C library is MT-safe */
+#define gmtime_r(tp,tmp) (gmtime(tp)?(*(tmp)=*gmtime(tp),(tmp)):0)
 #endif
 
 /** This is the toplevel directory where the timezone data is installed in. */
@@ -268,7 +275,7 @@ icaltimezone_get_vtimezone_properties	(icaltimezone	*zone,
 					 icalcomponent	*component)
 {
     icalproperty *prop;
-    const char *tzid;
+    const char *tzid, *tzname;
  
     prop = icalcomponent_get_first_property (component, ICAL_TZID_PROPERTY);
     if (!prop)
@@ -279,9 +286,17 @@ icaltimezone_get_vtimezone_properties	(icaltimezone	*zone,
     if (!tzid)
 	return 0;
 
+    prop = icalcomponent_get_first_property (component, ICAL_TZNAME_PROPERTY);
+    if (prop) {
+	tzname = icalproperty_get_tzname (prop);
+	zone->tznames = strdup(tzname);	
+    } else
+	zone->tznames = NULL;
+    
     zone->tzid = strdup (tzid);
     zone->component = component;
-	if ( zone->location != 0 ) free ( zone->location );
+    if ( zone->location != 0 )
+	free ( zone->location );
     zone->location = icaltimezone_get_location_from_vtimezone (component);
     zone->tznames = icaltimezone_get_tznames_from_vtimezone (component);
 
@@ -1358,6 +1373,72 @@ icaltimezone_get_builtin_timezone	(const char *location)
     return NULL;
 }
 
+static struct icaltimetype
+tm_to_icaltimetype (struct tm *tm)
+{
+	struct icaltimetype itt;
+
+	memset (&itt, 0, sizeof (struct icaltimetype));
+
+	itt.second = tm->tm_sec;
+	itt.minute = tm->tm_min;
+	itt.hour = tm->tm_hour;
+
+	itt.day = tm->tm_mday;
+	itt.month = tm->tm_mon + 1;
+	itt.year = tm->tm_year+ 1900;
+    
+	itt.is_utc = 0;
+	itt.is_date = 0; 
+	
+	return itt;
+}
+
+static int
+get_offset (icaltimezone *zone)
+{
+    struct tm local;
+    struct icaltimetype tt;
+    int offset;
+    time_t now = time(NULL);
+	
+    gmtime_r ((const time_t *) &now, &local);
+    tt = tm_to_icaltimetype (&local);
+    offset = icaltimezone_get_utc_offset(zone, &tt, NULL);
+
+    return offset;
+}
+
+/** Returns a single builtin timezone, given its offset from UTC */
+icaltimezone*
+icaltimezone_get_builtin_timezone_from_offset	(int offset, const char *tzname)
+{
+    icaltimezone *zone=NULL;
+    int cmp, count, i;
+    char *zone_location;
+
+    if (!builtin_timezones)
+	icaltimezone_init_builtin_timezones ();
+
+    if (offset==0)
+	return &utc_timezone;
+
+    count = builtin_timezones->num_elements;
+
+    for (i=0; i<count; i++) {
+	int z_offset;
+	zone = icalarray_element_at (builtin_timezones, i);
+	if (!zone->component)
+	    icaltimezone_load_builtin_timezone (zone);
+	
+	z_offset = get_offset(zone);
+
+	if (z_offset == offset && !strcmp(tzname, zone->tznames))
+	    return zone;
+    }
+    
+    return NULL;
+}
 
 /** Returns a single builtin timezone, given its TZID. */
 icaltimezone*
@@ -1564,8 +1645,6 @@ icaltimezone_load_builtin_timezone	(icaltimezone	*zone)
     icalparser_free (parser);
 	fclose (fp);
 
-	
-	
     /* Find the VTIMEZONE component inside the VCALENDAR. There should be 1. */
     subcomp = icalcomponent_get_first_component (comp,
 						 ICAL_VTIMEZONE_COMPONENT);
